@@ -24,6 +24,9 @@
 
 #include "xstuff.h"
 
+static int (* xstuff_old_xio_error_handler) (Display *) = NULL;
+static gboolean xstuff_display_is_dead = FALSE;
+
 static Atom
 panel_atom_get (const char *atom_name)
 {
@@ -90,19 +93,19 @@ get_typed_property_data (Display *xdisplay,
       requested_type != AnyPropertyType &&
       requested_type != type_returned)
     {
-      g_warning (G_GNUC_PRETTY_FUNCTION "(): Property has wrong type, probably on crack");
+      g_warning ("%s(): Property has wrong type, probably on crack", G_STRFUNC);
       abort++;
     }
   if (!abort && bytes_after_return)
     {
-      g_warning (G_GNUC_PRETTY_FUNCTION "(): Eeek, property has more than %u bytes, stored on harddisk?",
-		 prop_buffer_lengh);
+      g_warning ("%s(): Eeek, property has more than %u bytes, stored on harddisk?",
+		 G_STRFUNC, prop_buffer_lengh);
       abort++;
     }
   if (!abort && expected_format && expected_format != format_returned)
     {
-      g_warning (G_GNUC_PRETTY_FUNCTION "(): Expected format (%u) unmatched (%d), programmer was drunk?",
-		 expected_format, format_returned);
+      g_warning ("%s(): Expected format (%u) unmatched (%d), programmer was drunk?",
+		 G_STRFUNC, expected_format, format_returned);
       abort++;
     }
   if (!abort && prop_data && nitems_return && format_returned)
@@ -183,37 +186,6 @@ xstuff_net_wm_supports (const char *hint)
 	return gdk_net_wm_supports (gdk_atom_intern (hint, FALSE));
 }
 
-void
-xstuff_set_no_group (GdkWindow *win)
-{
-	XWMHints *old_wmhints;
-	XWMHints wmhints = {0};
-
-	XDeleteProperty (GDK_WINDOW_XDISPLAY (win),
-			 GDK_WINDOW_XWINDOW (win),
-			 panel_atom_get ("WM_CLIENT_LEADER"));
-
-	old_wmhints = XGetWMHints (GDK_WINDOW_XDISPLAY (win),
-				   GDK_WINDOW_XWINDOW (win));
-	/* General paranoia */
-	if (old_wmhints != NULL) {
-		memcpy (&wmhints, old_wmhints, sizeof (XWMHints));
-		XFree (old_wmhints);
-
-		wmhints.flags &= ~WindowGroupHint;
-		wmhints.window_group = 0;
-	} else {
-		/* General paranoia */
-		wmhints.flags = StateHint;
-		wmhints.window_group = 0;
-		wmhints.initial_state = NormalState;
-	}
-
-	XSetWMHints (GDK_WINDOW_XDISPLAY (win),
-		     GDK_WINDOW_XWINDOW (win),
-		     &wmhints);
-}
-
 /* This is such a broken stupid function. */   
 void
 xstuff_set_pos_size (GdkWindow *window, int x, int y, int w, int h)
@@ -323,6 +295,7 @@ draw_zoom_animation (GdkScreen *gscreen,
 {
 #define FRAMES (MINIATURIZE_ANIMATION_FRAMES_Z)
 	float cx[FRAMES], cy[FRAMES], cw[FRAMES], ch[FRAMES];
+	int cxi[FRAMES], cyi[FRAMES], cwi[FRAMES], chi[FRAMES];
 	float xstep, ystep, wstep, hstep;
 	int i, j;
 	GC frame_gc;
@@ -370,12 +343,15 @@ draw_zoom_animation (GdkScreen *gscreen,
 		cy[j] = (float)y;
 		cw[j] = (float)w;
 		ch[j] = (float)h;
+		cxi[j] = (int)cx[j];
+		cyi[j] = (int)cy[j];
+		cwi[j] = (int)cw[j];
+		chi[j] = (int)ch[j];
 	}
 	XGrabServer(dpy);
 	for (i=0; i<steps; i++) {
 		for (j=0; j<FRAMES; j++) {
-			XDrawRectangle(dpy, root_win, frame_gc, 
-				       (int)cx[j], (int)cy[j], (int)cw[j], (int)ch[j]);
+			XDrawRectangle(dpy, root_win, frame_gc, cxi[j], cyi[j], cwi[j], chi[j]);
 		}
 		XFlush(dpy);
 #if (MINIATURIZE_ANIMATION_DELAY_Z > 0)
@@ -385,24 +361,35 @@ draw_zoom_animation (GdkScreen *gscreen,
 #endif
 		for (j=0; j<FRAMES; j++) {
 			XDrawRectangle(dpy, root_win, frame_gc, 
-				       (int)cx[j], (int)cy[j], (int)cw[j], (int)ch[j]);
+				       cxi[j], cyi[j], cwi[j], chi[j]);
 			if (j<FRAMES-1) {
 				cx[j]=cx[j+1];
 				cy[j]=cy[j+1];
 				cw[j]=cw[j+1];
 				ch[j]=ch[j+1];
+				
+				cxi[j]=cxi[j+1];
+				cyi[j]=cyi[j+1];
+				cwi[j]=cwi[j+1];
+				chi[j]=chi[j+1];
+				
 			} else {
 				cx[j]+=xstep;
 				cy[j]+=ystep;
 				cw[j]+=wstep;
 				ch[j]+=hstep;
+
+				cxi[j] = (int)cx[j];
+				cyi[j] = (int)cy[j];
+				cwi[j] = (int)cw[j];
+				chi[j] = (int)ch[j];
 			}
 		}
 	}
 
 	for (j=0; j<FRAMES; j++) {
 		XDrawRectangle(dpy, root_win, frame_gc, 
-			       (int)cx[j], (int)cy[j], (int)cw[j], (int)ch[j]);
+				       cxi[j], cyi[j], cwi[j], chi[j]);
 	}
 	XFlush(dpy);
 #if (MINIATURIZE_ANIMATION_DELAY_Z > 0)
@@ -412,7 +399,7 @@ draw_zoom_animation (GdkScreen *gscreen,
 #endif
 	for (j=0; j<FRAMES; j++) {
 		XDrawRectangle(dpy, root_win, frame_gc, 
-			       (int)cx[j], (int)cy[j], (int)cw[j], (int)ch[j]);
+				       cxi[j], cyi[j], cwi[j], chi[j]);
 	}
     
 	XUngrabServer(dpy);
@@ -517,3 +504,26 @@ xstuff_grab_key_on_all_screens (int      keycode,
 				    gdk_x11_drawable_get_xid (root));
 	}
 }
+
+static int
+xstuff_xio_error_handler (Display *display)
+{
+	xstuff_display_is_dead = TRUE;
+
+	if (xstuff_old_xio_error_handler)
+		return xstuff_old_xio_error_handler (display);
+
+	return 0;
+}
+
+gboolean
+xstuff_is_display_dead (void)
+{
+	return xstuff_display_is_dead;
+}
+
+void
+xstuff_init (void)
+{
+	xstuff_old_xio_error_handler = XSetIOErrorHandler (xstuff_xio_error_handler);
+}
